@@ -28,6 +28,8 @@
 #include <NetworkManager.h>
 
 #include "list-box-helper.h"
+#include "ce-ip-address-entry.h"
+#include "ce-netmask-entry.h"
 #include "ce-page.h"
 #include "ce-page-ip4.h"
 #include "ui-helpers.h"
@@ -48,6 +50,7 @@ struct _CEPageIP4
         GtkRadioButton    *disabled_radio;
         GtkEntry          *dns_entry;
         GtkRadioButton    *local_radio;
+        GtkGrid           *main_box;
         GtkRadioButton    *manual_radio;
         GtkCheckButton    *never_default_check;
         GtkBox            *routes_box;
@@ -108,11 +111,11 @@ method_changed (CEPageIP4 *self)
 static void
 update_row_sensitivity (CEPageIP4 *self, GtkWidget *list)
 {
-        GList *children, *l;
+        g_autoptr(GList) children = NULL;
         gint rows = 0, i = 0;
 
         children = gtk_container_get_children (GTK_CONTAINER (list));
-        for (l = children; l; l = l->next) {
+        for (GList *l = children; l; l = l->next) {
                 GtkWidget *row = l->data;
                 GtkWidget *button;
 
@@ -120,7 +123,7 @@ update_row_sensitivity (CEPageIP4 *self, GtkWidget *list)
                 if (button != NULL)
                         rows++;
         }
-        for (l = children; l; l = l->next) {
+        for (GList *l = children; l; l = l->next) {
                 GtkWidget *row = l->data;
                 GtkWidget *button;
 
@@ -128,17 +131,16 @@ update_row_sensitivity (CEPageIP4 *self, GtkWidget *list)
                 if (button != NULL)
                         gtk_widget_set_sensitive (button, rows > 1 && ++i < rows);
         }
-        g_list_free (children);
 }
 
 static void
 update_row_gateway_sensitivity (CEPageIP4 *self)
 {
-        GList *children, *l;
+        g_autoptr(GList) children = NULL;
         gint rows = 0;
 
         children = gtk_container_get_children (GTK_CONTAINER (self->address_list));
-        for (l = children; l; l = l->next) {
+        for (GList *l = children; l; l = l->next) {
                 GtkWidget *row = l->data;
                 GtkWidget *entry;
 
@@ -148,7 +150,6 @@ update_row_gateway_sensitivity (CEPageIP4 *self)
 
                 rows++;
         }
-        g_list_free (children);
 }
 
 static void
@@ -175,22 +176,20 @@ static gboolean
 validate_row (GtkWidget *row)
 {
         GtkWidget *box;
-        GList *children, *l;
+        g_autoptr(GList) children = NULL;
         gboolean valid;
 
         valid = FALSE;
         box = gtk_bin_get_child (GTK_BIN (row));
         children = gtk_container_get_children (GTK_CONTAINER (box));
 
-        for (l = children; l != NULL; l = l->next) {
+        for (GList *l = children; l != NULL; l = l->next) {
                 if (!GTK_IS_ENTRY (l->data))
                         continue;
 
                 valid = valid || gtk_entry_get_text_length (l->data) > 0;
         }
 
-        g_list_free (children);
-
         return valid;
 }
 
@@ -211,7 +210,7 @@ add_address_row (CEPageIP4   *self,
         row_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
         gtk_style_context_add_class (gtk_widget_get_style_context (row_box), "linked");
 
-        widget = gtk_entry_new ();
+        widget = GTK_WIDGET (ce_ip_address_entry_new (AF_INET));
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_address_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "address", widget);
@@ -220,16 +219,16 @@ add_address_row (CEPageIP4   *self,
         gtk_widget_set_hexpand (widget, TRUE);
         gtk_container_add (GTK_CONTAINER (row_box), widget);
 
-        widget = gtk_entry_new ();
+        widget = GTK_WIDGET (ce_netmask_entry_new ());
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_address_row), self, G_CONNECT_SWAPPED);
-        g_object_set_data (G_OBJECT (row), "network", widget);
+        g_object_set_data (G_OBJECT (row), "netmask", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), network);
         gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
         gtk_widget_set_hexpand (widget, TRUE);
         gtk_container_add (GTK_CONTAINER (row_box), widget);
 
-        widget = gtk_entry_new ();
+        widget = GTK_WIDGET (ce_ip_address_entry_new (AF_INET));
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_address_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "gateway", widget);
@@ -261,19 +260,15 @@ add_address_row (CEPageIP4   *self,
 static void
 ensure_empty_address_row (CEPageIP4 *self)
 {
-        GList *children, *l;
+        g_autoptr(GList) children = NULL;
+        GList *l;
 
         children = gtk_container_get_children (GTK_CONTAINER (self->address_list));
-        l = children;
-
-        while (l && l->next)
-                l = l->next;
+        l = g_list_last (children);
 
         /* Add the last, stub row if needed*/
         if (!l || validate_row (l->data))
                 add_address_row (self, "", "", "");
-
-        g_list_free (children);
 }
 
 static void
@@ -357,7 +352,7 @@ add_route_row (CEPageIP4   *self,
         row_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
         gtk_style_context_add_class (gtk_widget_get_style_context (row_box), "linked");
 
-        widget = gtk_entry_new ();
+        widget = GTK_WIDGET (ce_ip_address_entry_new (AF_INET));
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "address", widget);
@@ -366,7 +361,7 @@ add_route_row (CEPageIP4   *self,
         gtk_widget_set_hexpand (widget, TRUE);
         gtk_container_add (GTK_CONTAINER (row_box), widget);
 
-        widget = gtk_entry_new ();
+        widget = GTK_WIDGET (ce_netmask_entry_new ());
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "netmask", widget);
@@ -375,7 +370,7 @@ add_route_row (CEPageIP4   *self,
         gtk_widget_set_hexpand (widget, TRUE);
         gtk_container_add (GTK_CONTAINER (row_box), widget);
 
-        widget = gtk_entry_new ();
+        widget = GTK_WIDGET (ce_ip_address_entry_new (AF_INET));
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "gateway", widget);
@@ -421,19 +416,15 @@ add_route_row (CEPageIP4   *self,
 static void
 ensure_empty_routes_row (CEPageIP4 *self)
 {
-        GList *children, *l;
+        g_autoptr(GList) children = NULL;
+        GList *l;
 
         children = gtk_container_get_children (GTK_CONTAINER (self->routes_list));
-        l = children;
-
-        while (l && l->next)
-                l = l->next;
+        l = g_list_last (children);
 
         /* Add the last, stub row if needed*/
         if (!l || validate_row (l->data))
                 add_route_row (self, "", "", "", -1);
-
-        g_list_free (children);
 }
 
 static void
@@ -479,6 +470,9 @@ connect_ip4_page (CEPageIP4 *self)
         const gchar *str_method;
         guint method;
 
+        gtk_container_set_focus_vadjustment (GTK_CONTAINER (self->main_box),
+                                             gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self)));
+
         add_address_box (self);
         add_dns_section (self);
         add_routes_box (self);
@@ -537,32 +531,6 @@ connect_ip4_page (CEPageIP4 *self)
         method_changed (self);
 }
 
-static gboolean
-parse_netmask (const char *str, guint32 *prefix)
-{
-        struct in_addr tmp_addr;
-        glong tmp_prefix;
-
-        errno = 0;
-
-        /* Is it a prefix? */
-        if (!strchr (str, '.')) {
-                tmp_prefix = strtol (str, NULL, 10);
-                if (!errno && tmp_prefix >= 0 && tmp_prefix <= 32) {
-                        *prefix = tmp_prefix;
-                        return TRUE;
-                }
-        }
-
-        /* Is it a netmask? */
-        if (inet_pton (AF_INET, str, &tmp_addr) > 0) {
-                *prefix = nm_utils_ip4_netmask_to_prefix (tmp_addr.s_addr);
-                return TRUE;
-        }
-
-        return FALSE;
-}
-
 static gboolean
 ui_to_setting (CEPageIP4 *self)
 {
@@ -574,7 +542,8 @@ ui_to_setting (CEPageIP4 *self)
         GPtrArray *dns_servers = NULL;
         GPtrArray *routes = NULL;
         GStrv dns_addresses = NULL;
-        GList *children, *l;
+        g_autoptr(GList) address_children = NULL;
+        g_autoptr(GList) routes_children = NULL;
         gboolean ret = TRUE;
         const char *default_gateway = NULL;
         gchar *dns_text = NULL;
@@ -593,76 +562,52 @@ ui_to_setting (CEPageIP4 *self)
 
         addresses = g_ptr_array_new_with_free_func ((GDestroyNotify) nm_ip_address_unref);
         if (g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL))
-                children = gtk_container_get_children (GTK_CONTAINER (self->address_list));
-        else
-                children = NULL;
+                address_children = gtk_container_get_children (GTK_CONTAINER (self->address_list));
 
-        for (l = children; l; l = l->next) {
+        for (GList *l = address_children; l; l = l->next) {
                 GtkWidget *row = l->data;
-                GtkEntry *entry;
-                GtkEntry *gateway_entry;
-                const gchar *text_address;
-                const gchar *text_netmask;
-                const gchar *text_gateway = "";
+                CEIPAddressEntry *address_entry;
+                CENetmaskEntry *netmask_entry;
+                CEIPAddressEntry *gateway_entry;
                 NMIPAddress *addr;
-                guint32 prefix;
 
-                entry = GTK_ENTRY (g_object_get_data (G_OBJECT (row), "address"));
-                if (!entry)
+                address_entry = CE_IP_ADDRESS_ENTRY (g_object_get_data (G_OBJECT (row), "address"));
+                if (!address_entry)
                         continue;
 
-                text_address = gtk_entry_get_text (entry);
-                text_netmask = gtk_entry_get_text (GTK_ENTRY (g_object_get_data (G_OBJECT (row), "network")));
-                gateway_entry = g_object_get_data (G_OBJECT (row), "gateway");
-                if (gtk_widget_is_visible (GTK_WIDGET (gateway_entry)))
-                        text_gateway = gtk_entry_get_text (gateway_entry);
+                netmask_entry = CE_NETMASK_ENTRY (g_object_get_data (G_OBJECT (row), "netmask"));
+                gateway_entry = CE_IP_ADDRESS_ENTRY (g_object_get_data (G_OBJECT (row), "gateway"));
 
-                if (!*text_address && !*text_netmask && !*text_gateway) {
+                if (ce_ip_address_entry_is_empty (address_entry) && ce_netmask_entry_is_empty (netmask_entry) && ce_ip_address_entry_is_empty (gateway_entry)) {
                         /* ignore empty rows */
-                        widget_unset_error (GTK_WIDGET (entry));
-                        widget_unset_error (g_object_get_data (G_OBJECT (row), "network"));
-                        widget_unset_error (GTK_WIDGET (gateway_entry));
                         continue;
                 }
 
-                if (!nm_utils_ipaddr_valid (AF_INET, text_address)) {
-                        widget_set_error (GTK_WIDGET (entry));
+                if (!ce_ip_address_entry_is_valid (address_entry))
                         ret = FALSE;
-                } else {
-                        widget_unset_error (GTK_WIDGET (entry));
-                }
 
-                if (!parse_netmask (text_netmask, &prefix)) {
-                        widget_set_error (g_object_get_data (G_OBJECT (row), "network"));
+                if (!ce_netmask_entry_is_valid (netmask_entry))
                         ret = FALSE;
-                } else {
-                        widget_unset_error (g_object_get_data (G_OBJECT (row), "network"));
-                }
 
-                if (gtk_widget_is_visible (GTK_WIDGET (gateway_entry)) &&
-                    *text_gateway &&
-                    !nm_utils_ipaddr_valid (AF_INET, text_gateway)) {
-                        widget_set_error (g_object_get_data (G_OBJECT (row), "gateway"));
+                if (!ce_ip_address_entry_is_valid (gateway_entry)) {
                         ret = FALSE;
                 } else {
-                         widget_unset_error (GTK_WIDGET (gateway_entry));
-                         if (gtk_widget_is_visible (GTK_WIDGET (gateway_entry)) && *text_gateway) {
+                         if (!ce_ip_address_entry_is_empty (gateway_entry)) {
                                  g_assert (default_gateway == NULL);
-                                 default_gateway = text_gateway;
+                                 default_gateway = gtk_entry_get_text (GTK_ENTRY (gateway_entry));
                          }
                 }
 
                 if (!ret)
                         continue;
 
-                addr = nm_ip_address_new (AF_INET, text_address, prefix, NULL);
+                addr = nm_ip_address_new (AF_INET, gtk_entry_get_text (GTK_ENTRY (address_entry)), ce_netmask_entry_get_prefix (netmask_entry), NULL);
                 if (addr)
                         g_ptr_array_add (addresses, addr);
 
                 if (!l || !l->next)
                         ensure_empty_address_row (self);
         }
-        g_list_free (children);
 
         if (addresses->len == 0) {
                 g_ptr_array_free (addresses, TRUE);
@@ -707,55 +652,38 @@ ui_to_setting (CEPageIP4 *self)
         routes = g_ptr_array_new_with_free_func ((GDestroyNotify) nm_ip_route_unref);
         if (g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO) ||
             g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL))
-                children = gtk_container_get_children (GTK_CONTAINER (self->routes_list));
-        else
-                children = NULL;
+                routes_children = gtk_container_get_children (GTK_CONTAINER (self->routes_list));
 
-        for (l = children; l; l = l->next) {
+        for (GList *l = routes_children; l; l = l->next) {
                 GtkWidget *row = l->data;
-                GtkEntry *entry;
-                const gchar *text_address;
-                const gchar *text_netmask;
-                const gchar *text_gateway;
+                CEIPAddressEntry *address_entry;
+                CENetmaskEntry *netmask_entry;
+                CEIPAddressEntry *gateway_entry;
                 const gchar *text_metric;
                 gint64 metric;
-                guint32 netmask;
                 NMIPRoute *route;
 
-                entry = GTK_ENTRY (g_object_get_data (G_OBJECT (row), "address"));
-                if (!entry)
+                address_entry = CE_IP_ADDRESS_ENTRY (g_object_get_data (G_OBJECT (row), "address"));
+                if (!address_entry)
                         continue;
 
-                text_address = gtk_entry_get_text (entry);
-                text_netmask = gtk_entry_get_text (GTK_ENTRY (g_object_get_data (G_OBJECT (row), "netmask")));
-                text_gateway = gtk_entry_get_text (GTK_ENTRY (g_object_get_data (G_OBJECT (row), "gateway")));
+                netmask_entry = CE_NETMASK_ENTRY (g_object_get_data (G_OBJECT (row), "netmask"));
+                gateway_entry = CE_IP_ADDRESS_ENTRY (g_object_get_data (G_OBJECT (row), "gateway"));
                 text_metric = gtk_entry_get_text (GTK_ENTRY (g_object_get_data (G_OBJECT (row), "metric")));
 
-                if (!*text_address && !*text_netmask && !*text_gateway && !*text_metric) {
+                if (ce_ip_address_entry_is_empty (address_entry) && ce_netmask_entry_is_empty (netmask_entry) && ce_ip_address_entry_is_empty (gateway_entry) && !*text_metric) {
                         /* ignore empty rows */
                         continue;
                 }
 
-                if (text_address && !nm_utils_ipaddr_valid (AF_INET, text_address)) {
-                        widget_set_error (GTK_WIDGET (entry));
+                if (!ce_ip_address_entry_is_valid (address_entry))
                         ret = FALSE;
-                } else {
-                        widget_unset_error (GTK_WIDGET (entry));
-                }
 
-                if (!parse_netmask (text_netmask, &netmask)) {
-                        widget_set_error (GTK_WIDGET (g_object_get_data (G_OBJECT (row), "netmask")));
+                if (!ce_netmask_entry_is_valid (netmask_entry))
                         ret = FALSE;
-                } else {
-                        widget_unset_error (GTK_WIDGET (g_object_get_data (G_OBJECT (row), "netmask")));
-                }
 
-                if (text_gateway && !nm_utils_ipaddr_valid (AF_INET, text_gateway)) {
-                        widget_set_error (GTK_WIDGET (g_object_get_data (G_OBJECT (row), "gateway")));
+                if (!ce_ip_address_entry_is_valid (gateway_entry))
                         ret = FALSE;
-                } else {
-                        widget_unset_error (GTK_WIDGET (g_object_get_data (G_OBJECT (row), "gateway")));
-                }
 
                 metric = -1;
                 if (*text_metric) {
@@ -774,14 +702,13 @@ ui_to_setting (CEPageIP4 *self)
                 if (!ret)
                         continue;
 
-                route = nm_ip_route_new (AF_INET, text_address, netmask, text_gateway, metric, NULL);
+                route = nm_ip_route_new (AF_INET, gtk_entry_get_text (GTK_ENTRY (address_entry)), ce_netmask_entry_get_prefix (netmask_entry), gtk_entry_get_text (GTK_ENTRY (gateway_entry)), metric, NULL);
                 if (route)
                         g_ptr_array_add (routes, route);
 
                 if (!l || !l->next)
                         ensure_empty_routes_row (self);
         }
-        g_list_free (children);
 
         if (routes->len == 0) {
                 g_ptr_array_free (routes, TRUE);
@@ -860,6 +787,7 @@ ce_page_ip4_class_init (CEPageIP4Class *klass)
         gtk_widget_class_bind_template_child (widget_class, CEPageIP4, disabled_radio);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP4, dns_entry);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP4, local_radio);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, main_box);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP4, manual_radio);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP4, never_default_check);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP4, routes_box);
