@@ -59,7 +59,6 @@ struct _NetConnectionEditor
         GtkNotebook      *notebook;
         GtkStack         *toplevel_stack;
 
-        GtkWidget        *parent_window;
         NMClient         *client;
         NMDevice         *device;
 
@@ -70,12 +69,10 @@ struct _NetConnectionEditor
         NMAccessPoint    *ap;
 
         GSList *initializing_pages;
-        GSList *pages;
 
         NMClientPermissionResult can_modify;
 
         gboolean          title_set;
-        gboolean          show_when_initialized;
 };
 
 G_DEFINE_TYPE (NetConnectionEditor, net_connection_editor, GTK_TYPE_DIALOG)
@@ -192,14 +189,9 @@ static void
 net_connection_editor_finalize (GObject *object)
 {
         NetConnectionEditor *self = NET_CONNECTION_EDITOR (object);
-        GSList *l;
-
-        for (l = self->pages; l != NULL; l = l->next)
-                g_signal_handlers_disconnect_by_func (l->data, page_changed, self);
 
         g_clear_object (&self->connection);
         g_clear_object (&self->orig_connection);
-        g_clear_object (&self->parent_window);
         g_clear_object (&self->device);
         g_clear_object (&self->client);
         g_clear_object (&self->ap);
@@ -250,7 +242,7 @@ net_connection_editor_error_dialog (NetConnectionEditor *self,
         if (gtk_widget_is_visible (GTK_WIDGET (self)))
                 parent = GTK_WINDOW (self);
         else
-                parent = GTK_WINDOW (self->parent_window);
+                parent = gtk_window_get_transient_for (GTK_WINDOW (self));
 
         dialog = gtk_message_dialog_new (parent,
                                          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -331,7 +323,7 @@ update_sensitivity (NetConnectionEditor *self)
 {
         NMSettingConnection *sc;
         gboolean sensitive;
-        GSList *l;
+        GList *pages;
 
         if (!editor_is_initialized (self))
                 return;
@@ -344,29 +336,34 @@ update_sensitivity (NetConnectionEditor *self)
                 sensitive = self->can_modify;
         }
 
-        for (l = self->pages; l; l = l->next)
-                gtk_widget_set_sensitive (GTK_WIDGET (l->data), sensitive);
+        pages = gtk_container_get_children (GTK_CONTAINER (self->notebook));
+        for (GList *l = pages; l; l = l->next) {
+                CEPage *page = l->data;
+                gtk_widget_set_sensitive (GTK_WIDGET (page), sensitive);
+        }
 }
 
 static void
 validate (NetConnectionEditor *self)
 {
         gboolean valid = FALSE;
-        GSList *l;
+        GList *pages;
 
         if (!editor_is_initialized (self))
                 goto done;
 
         valid = TRUE;
-        for (l = self->pages; l; l = l->next) {
+        pages = gtk_container_get_children (GTK_CONTAINER (self->notebook));
+        for (GList *l = pages; l; l = l->next) {
+                CEPage *page = l->data;
                 g_autoptr(GError) error = NULL;
 
-                if (!ce_page_validate (CE_PAGE (l->data), self->connection, &error)) {
+                if (!ce_page_validate (page, self->connection, &error)) {
                         valid = FALSE;
                         if (error) {
-                                g_debug ("Invalid setting %s: %s", ce_page_get_title (CE_PAGE (l->data)), error->message);
+                                g_debug ("Invalid setting %s: %s", ce_page_get_title (page), error->message);
                         } else {
-                                g_debug ("Invalid setting %s", ce_page_get_title (CE_PAGE (l->data)));
+                                g_debug ("Invalid setting %s", ce_page_get_title (page));
                         }
                 }
         }
@@ -398,11 +395,9 @@ recheck_initialization (NetConnectionEditor *self)
         if (!editor_is_initialized (self))
                 return;
 
+        gtk_stack_set_visible_child (self->toplevel_stack, GTK_WIDGET (self->notebook));
         gtk_notebook_set_current_page (self->notebook, 0);
 
-        if (self->show_when_initialized)
-                gtk_window_present (GTK_WINDOW (self));
-
         g_idle_add (idle_validate, self);
 }
 
@@ -429,7 +424,6 @@ page_initialized (NetConnectionEditor *self, GError *error, CEPage *page)
         gtk_notebook_insert_page (self->notebook, GTK_WIDGET (page), label, i);
 
         self->initializing_pages = g_slist_remove (self->initializing_pages, page);
-        self->pages = g_slist_append (self->pages, page);
 
         recheck_initialization (self);
 }
@@ -786,8 +780,7 @@ permission_changed (NetConnectionEditor      *self,
 }
 
 NetConnectionEditor *
-net_connection_editor_new (GtkWindow        *parent_window,
-                           NMConnection     *connection,
+net_connection_editor_new (NMConnection     *connection,
                            NMDevice         *device,
                            NMAccessPoint    *ap,
                            NMClient         *client)
@@ -799,11 +792,6 @@ net_connection_editor_new (GtkWindow        *parent_window,
                              "use-header-bar", 1,
                              NULL);
 
-        if (parent_window) {
-                self->parent_window = GTK_WIDGET (g_object_ref (parent_window));
-                gtk_window_set_transient_for (GTK_WINDOW (self),
-                                              parent_window);
-        }
         if (ap)
                 self->ap = g_object_ref (ap);
         if (device)
@@ -822,16 +810,6 @@ net_connection_editor_new (GtkWindow        *parent_window,
         return self;
 }
 
-void
-net_connection_editor_run (NetConnectionEditor *self)
-{
-        if (!editor_is_initialized (self)) {
-                self->show_when_initialized = TRUE;
-                return;
-        }
-        gtk_window_present (GTK_WINDOW (self));
-}
-
 static void
 forgotten_cb (GObject *source_object,
               GAsyncResult *res,
