@@ -40,8 +40,8 @@ wireless_dialog_closure_closure_notify (gpointer data,
                                         GClosure *gclosure)
 {
         WirelessDialogClosure *closure = data;
-        g_object_unref (closure->client);
 
+        g_clear_object (&closure->client);
         g_slice_free (WirelessDialogClosure, data);
 }
 
@@ -49,9 +49,9 @@ static void
 mobile_dialog_closure_free (gpointer data)
 {
         MobileDialogClosure *closure = data;
-        g_object_unref (closure->client);
-        g_object_unref (closure->device);
 
+        g_clear_object (&closure->client);
+        g_clear_object (&closure->device);
         g_slice_free (MobileDialogClosure, data);
 }
 
@@ -76,12 +76,10 @@ activate_existing_cb (GObject *source_object,
                       GAsyncResult *res,
                       gpointer user_data)
 {
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
 
-        if (!nm_client_activate_connection_finish (NM_CLIENT (source_object), res, &error)) {
+        if (!nm_client_activate_connection_finish (NM_CLIENT (source_object), res, &error))
 		g_warning ("Failed to activate connection: (%d) %s", error->code, error->message);
-		g_error_free (error);
-	}
 }
 
 static void
@@ -89,25 +87,10 @@ activate_new_cb (GObject *source_object,
                  GAsyncResult *res,
                  gpointer user_data)
 {
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
 
-        if (!nm_client_add_and_activate_connection_finish (NM_CLIENT (source_object), res, &error)) {
+        if (!nm_client_add_and_activate_connection_finish (NM_CLIENT (source_object), res, &error))
 		g_warning ("Failed to add new connection: (%d) %s", error->code, error->message);
-		g_error_free (error);
-	}
-}
-
-static void
-nag_dialog_response_cb (GtkDialog *nag_dialog,
-                        gint response,
-                        gpointer user_data)
-{
-	NMAWifiDialog *wireless_dialog = NMA_WIFI_DIALOG (user_data);
-
-	if (response == GTK_RESPONSE_NO) {  /* user opted not to correct the warning */
-		nma_wifi_dialog_set_nag_ignored (wireless_dialog, TRUE);
-		gtk_dialog_response (GTK_DIALOG (wireless_dialog), GTK_RESPONSE_OK);
-	}
 }
 
 static void
@@ -117,7 +100,8 @@ wireless_dialog_response_cb (GtkDialog *foo,
 {
 	NMAWifiDialog *dialog = NMA_WIFI_DIALOG (foo);
 	WirelessDialogClosure *closure = user_data;
-	NMConnection *connection, *fuzzy_match = NULL;
+	g_autoptr(NMConnection) connection = NULL;
+	NMConnection *fuzzy_match = NULL;
 	NMDevice *device;
 	NMAccessPoint *ap;
 	const GPtrArray *all;
@@ -126,22 +110,6 @@ wireless_dialog_response_cb (GtkDialog *foo,
 	if (response != GTK_RESPONSE_OK)
 		goto done;
 
-	if (!nma_wifi_dialog_get_nag_ignored (dialog)) {
-		GtkWidget *nag_dialog;
-
-		/* Nag the user about certificates or whatever.  Only destroy the dialog
-		 * if no nagging was done.
-		 */
-		nag_dialog = nma_wifi_dialog_nag_user (dialog);
-		if (nag_dialog) {
-			gtk_window_set_transient_for (GTK_WINDOW (nag_dialog), GTK_WINDOW (dialog));
-			g_signal_connect (nag_dialog, "response",
-			                  G_CALLBACK (nag_dialog_response_cb),
-			                  dialog);
-			return;
-		}
-	}
-
 	/* nma_wifi_dialog_get_connection() returns a connection with the
 	 * refcount incremented, so the caller must remember to unref it.
 	 */
@@ -197,9 +165,6 @@ wireless_dialog_response_cb (GtkDialog *foo,
 		                                             NULL);
 	}
 
-	/* Balance nma_wifi_dialog_get_connection() */
-	g_object_unref (connection);
-
 done:
 	gtk_widget_hide (GTK_WIDGET (dialog));
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -263,7 +228,7 @@ cc_network_panel_connect_to_8021x_network (GtkWidget        *toplevel,
 	NMSetting8021x *s_8021x;
 	NM80211ApSecurityFlags wpa_flags, rsn_flags;
 	GtkWidget *dialog;
-	char *uuid;
+	g_autofree gchar *uuid = NULL;
         NMAccessPoint *ap;
 
         g_debug ("connect to 8021x wifi");
@@ -291,7 +256,6 @@ cc_network_panel_connect_to_8021x_network (GtkWidget        *toplevel,
         s_con = (NMSettingConnection *) nm_setting_connection_new ();
         uuid = nm_utils_uuid_generate ();
         g_object_set (s_con, NM_SETTING_CONNECTION_UUID, uuid, NULL);
-        g_free (uuid);
         nm_connection_add_setting (connection, NM_SETTING (s_con));
 
         s_wifi = (NMSettingWireless *) nm_setting_wireless_new ();
@@ -348,7 +312,8 @@ cdma_mobile_wizard_done (NMAMobileWizard *wizard,
 
 	if (!canceled && method) {
 		NMSetting *setting;
-		char *uuid, *id;
+		g_autofree gchar *uuid = NULL;
+		g_autofree gchar *id = NULL;
 
 		if (method->devtype != NM_DEVICE_MODEM_CAPABILITY_CDMA_EVDO) {
 			g_warning ("Unexpected device type (not CDMA).");
@@ -390,8 +355,6 @@ cdma_mobile_wizard_done (NMAMobileWizard *wizard,
 		              NM_SETTING_CONNECTION_AUTOCONNECT, FALSE,
 		              NM_SETTING_CONNECTION_UUID, uuid,
 		              NULL);
-		g_free (uuid);
-		g_free (id);
 		nm_connection_add_setting (connection, setting);
 	}
 
@@ -410,7 +373,8 @@ gsm_mobile_wizard_done (NMAMobileWizard *wizard,
 
 	if (!canceled && method) {
 		NMSetting *setting;
-		char *uuid, *id;
+		g_autofree gchar *uuid = NULL;
+		g_autofree gchar *id = NULL;
 
 		if (method->devtype != NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS) {
 			g_warning ("Unexpected device type (not GSM).");
@@ -453,8 +417,6 @@ gsm_mobile_wizard_done (NMAMobileWizard *wizard,
 		              NM_SETTING_CONNECTION_AUTOCONNECT, FALSE,
 		              NM_SETTING_CONNECTION_UUID, uuid,
 		              NULL);
-		g_free (uuid);
-		g_free (id);
 		nm_connection_add_setting (connection, setting);
 	}
 
