@@ -230,7 +230,7 @@ hdy_header_group_child_new_for_header_group (HdyHeaderGroup *header_group)
   self->type = HDY_HEADER_GROUP_CHILD_TYPE_HEADER_GROUP;
   self->object = G_OBJECT (header_group);
 
-  g_object_weak_unref (G_OBJECT (header_group), (GWeakNotify) object_destroyed_cb, self);
+  g_object_weak_ref (G_OBJECT (header_group), (GWeakNotify) object_destroyed_cb, self);
 
   g_signal_connect_swapped (header_group, "update-decoration-layouts", G_CALLBACK (forward_update_decoration_layouts), self);
 
@@ -679,11 +679,11 @@ _gtk_builder_error_unhandled_tag (GtkBuilder           *builder,
 /* This has been copied and modified from gtksizegroup.c. */
 static void
 header_group_start_element (GMarkupParseContext  *context,
-                          const gchar          *element_name,
-                          const gchar         **names,
-                          const gchar         **values,
-                          gpointer              user_data,
-                          GError              **error)
+                            const gchar          *element_name,
+                            const gchar         **names,
+                            const gchar         **values,
+                            gpointer              user_data,
+                            GError              **error)
 {
   GSListSubParserData *data = (GSListSubParserData*)user_data;
 
@@ -730,7 +730,11 @@ header_group_start_element (GMarkupParseContext  *context,
 /* This has been copied and modified from gtksizegroup.c. */
 static const GMarkupParser header_group_parser =
   {
-    header_group_start_element
+    header_group_start_element,
+    NULL,
+    NULL,
+    NULL,
+    NULL
   };
 
 /* This has been copied and modified from gtksizegroup.c. */
